# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class AuthSchemeField(UncheckedBaseModel):
    """
    Auth scheme field.
    """

    name: str
    display_name: typing.Optional[str] = None
    description: str
    type: str
    default: typing.Optional[str] = None
    required: typing.Optional[bool] = None
    expected_from_customer: typing.Optional[bool] = None
    get_current_user_endpoint: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
